package algo

import core._

sealed trait AlgoBuiltin extends Builtin

case class IdInt32(input : IR) extends AlgoBuiltin {
  override val innerIR: IR = FunctionCall(BuiltinFunction(FunType(IntType(32), IntType(32))), input)
}

case class Id(input : IR, dt: TypeT = DataTypeVar()) extends AlgoBuiltin { // TODO is the dt really needed, or can I just use input.t (see also all the other Buildin Algos)
  override val innerIR: IR = {
    val dtv = DataTypeVar()
    FunctionCall(TypeFunctionCall(BuiltinTypeFunction(TypeFunType(dtv, FunType(dtv, dtv))), dt), input)
  }
}

case class IntCst(v: Int) extends AlgoBuiltin {
  override val innerIR: IR = BuiltinValue(IntType(v))
}

case class Reorder(input: IR, at: TypeT = ArrayTypeVar()) extends Builtin {
  override val innerIR: IR = {
    val atv = ArrayTypeVar()
    FunctionCall(TypeFunctionCall(BuiltinTypeFunction(TypeFunType(atv, FunType(atv, atv))), at), input)
  }
}

case class Split(input: IR, cs: IR /*chunk size*/, inseqt: TypeT = SeqTypeVar(), cst: TypeT = ArithTypeVar()) extends Builtin {
  override val innerIR: IR = {
    val atv = ArithTypeVar() // chunk size
    val seqtv = SeqTypeVar()
    // TODO typevars correct like this?
    FunctionCall(TypeFunctionCall(BuiltinTypeFunction(TypeFunType(Seq(seqtv, atv), FunType(Seq(seqtv, atv), SeqType(SeqType(seqtv.et, atv), seqtv.len.ae / atv)))), Seq(inseqt, cst)), Seq(input, cs))
  }
}

case class Join(input: IR, st: TypeT = SeqTypeVar(SeqTypeVar())) extends Builtin {
  override val innerIR: IR = {
    val stv_inner = SeqTypeVar()
    val stv_outer = SeqTypeVar(stv_inner)
    FunctionCall(TypeFunctionCall(BuiltinTypeFunction(TypeFunType(stv_outer, FunType(stv_outer, SeqType(stv_inner.et, stv_outer.len.ae * stv_inner.len.ae)))), st), input)
  }
}

// TODO: testing only (remove this one later)
case class CutInHalf(input: IR, st: TypeT = SeqTypeVar(SeqTypeVar())) extends Builtin {
  override val innerIR: IR = {
    val stv = SeqTypeVar()
    FunctionCall(TypeFunctionCall(BuiltinTypeFunction(TypeFunType(stv, FunType(stv, SeqType(stv.et, stv.len.ae / 2)))), st), input)
  }
}

case class Map(f: IR, inseq: IR, ft: TypeT = FunTypeVar(), inseqt: TypeT = SeqTypeVar()) extends Builtin {
  override val innerIR: IR = {
    val indtv = DataTypeVar()
    val outdtv = DataTypeVar()
    val ftv = FunTypeVar(indtv, outdtv)
    val atv = ArithTypeVar()
    val inseqtv = SeqTypeVar(indtv, atv)
    val outseqtv = SeqTypeVar(outdtv, atv)

    FunctionCall(TypeFunctionCall(BuiltinTypeFunction(TypeFunType(Seq(ftv, inseqtv), FunType(Seq(ftv, inseqtv), outseqtv))), Seq(ft, inseqt)), Seq(f, inseq))
  }
}
