package core

trait TreeNode[T <: TreeNode[T]] {
  def children: Seq[T]

  def build(children: Seq[T]): T

  def map(f: T => T): T = { // TODO should be final, but ArithTypeT must? override it...
    build(children.map(f(_)))
  }

  final def foreach(f: T => Unit): Unit = {
    children.foreach(f(_))
  }

  final def visit(prepost: T => T => Unit): Unit = {
    val post = prepost(this.asInstanceOf[T])
    foreach(_.visit(prepost))
    post(this.asInstanceOf[T])
  }

  final def visit(pre: T => Unit, post: T => Unit): Unit = {
    pre(this.asInstanceOf[T])
    foreach(_.visit(pre, post))
    post(this.asInstanceOf[T])
  }

  final def visitAndRebuild(prepost: T => (T, T => T)): T = {
    val (newType, post) = prepost(this.asInstanceOf[T])
    post(newType.map(_.visitAndRebuild(prepost)))
  }

  final def visitAndRebuild(pre: T => T, post: T => T): T = post(pre(this.asInstanceOf[T]).map(_.visitAndRebuild(pre, post)))
}
