package core

import algo._
import core.util.DotPrinter
import org.junit.Test

class TypeCheckerTest {

  @Test
  def testTypeIdFunction(): Unit = {
    val dtv = DataTypeVar()
    val id = Lambda(Param(dtv), Param(dtv))
    val tf = TypeLambda(dtv, id)
    val tree = FunctionCall(TypeFunctionCall(tf), Param(SeqType(IntType(13), ArithType(100))))
    typeCheck(tree)
  }

  @Test
  def testTypeIdAlgoFunction(): Unit = {
    val tree = Id(Param(IntType(13)))
    typeCheck(tree)
  }

  @Test
  def testLambda(): Unit = {
    val input = Param(IntType(4))
    val id = Lambda(input, input)
    val tree = FunctionCall(id, input)
    typeCheck(tree)
  }

  @Test
  def testMapIdFunction(): Unit = {
    val it = IntType(16)
    val at = ArithType(100)
    val inft = FunType(it, it)
    val inseqt = SeqType(it, at)
    val outseqt = SeqType(it, at)

    val tree = FunctionCall(BuiltinFunction(FunType(Seq(inft, inseqt), outseqt)), Seq(Param(inft), Param(inseqt)))
    typeCheck(tree)
  }

  @Test
  def testTypeMapIdFunction(): Unit = {
    val dt = DataTypeVar()
    val ft = FunType(dt, dt)
    val inseqt = SeqType(IntType(16), ArithType(100))

    val indtv = DataTypeVar()
    val outdtv = DataTypeVar()
    val ftv = FunTypeVar(indtv, outdtv)
    val atv = ArithTypeVar()
    val inseqtv = SeqTypeVar(indtv, atv)
    val outseqtv = SeqTypeVar(outdtv, atv)

    val tree = FunctionCall(TypeFunctionCall(BuiltinTypeFunction(TypeFunType(Seq(ftv, inseqtv), FunType(Seq(ftv, inseqtv), outseqtv))), Seq(ft, inseqt)), Seq(Param(ft), Param(inseqt)))
    typeCheck(tree)
  }

  @Test
  def testTypeMapIdAlgoFunction(): Unit = {
    val dtv = DataTypeVar()
    val ft = FunType(dtv, dtv)
    val inseqt = SeqType(IntType(16), ArithType(100))

    val tree = Map(Param(ft), Param(inseqt), ft, inseqt)
    typeCheck(tree)
  }

  @Test
  def testComplexArithTypeVarExpression(): Unit = {
    val input = Param(SeqType(IntType(32), 10))
    val tree = CutInHalf(input)
    typeCheck(tree)
  }

  @Test
  def testSimpleArithTypeVarSubstitution(): Unit = {
    val atNew = ArithType(40)
    val atOld: ArithTypeT = ArithType(ArithTypeVar() + 2)

    val transformed = atOld.visitAndRebuild(t => t, {
        case _: TypeVarT => atNew
        case t => t
    })
    
    assert(transformed == ArithType(42))
  }

  private def typeCheck(tree: IR): Unit = {
    val rebuiltTree = TypeChecker.rebuild(tree)

    DotPrinter.toFile(tree, "test.dot")
    //println(tree)
    DotPrinter.toFile(rebuiltTree, "test2.dot")
    //println(rebuiltTree)

    assert(!rebuiltTree.hasFreeTypeVar)
  }
}
